import { expect, test } from "bun:test";
import {
  PlatformKeyboardShortcut,
  getEventShortcutNames,
  getPlatformShortcutName,
} from "../shortcuts";

function createKeyboardEvent(
  eventName: string,
  options: Partial<KeyboardEventInit> = {}
): KeyboardEvent {
  const event = new KeyboardEvent(eventName, options) as any;
  // Fixes an issue in happy-dom where the keyCode isn't assigned
  event.keyCode = options.keyCode || 0;
  return event;
}

test("picks a shortcut key for the given platform", () => {
  const shortcut: PlatformKeyboardShortcut = {
    key: "k",
    mac: "m",
    win: "w",
  };

  expect(getPlatformShortcutName(shortcut, "key")).toEqual("k");
  expect(getPlatformShortcutName(shortcut, "mac")).toEqual("m");
  expect(getPlatformShortcutName(shortcut, "win")).toEqual("w");
  expect(getPlatformShortcutName(shortcut, "linux")).toEqual("k");
});

test("supports platform-specific commands", () => {
  const shortcut: PlatformKeyboardShortcut = {
    mac: "m",
  };

  expect(getPlatformShortcutName(shortcut, "mac")).toEqual("m");
  expect(getPlatformShortcutName(shortcut, "win")).toEqual(undefined as any);
});

test("gets event shortcut name", () => {
  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "d",
        keyCode: 100,
        shiftKey: false,
      }),
      "mac"
    )
  ).toEqual(["d"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "D",
        keyCode: 68,
        shiftKey: true,
      }),
      "mac"
    )
  ).toEqual(["D", "Shift-d"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "2",
        keyCode: 50,
      }),
      "mac"
    )
  ).toEqual(["2"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "2",
        keyCode: 50,
        shiftKey: true,
      }),
      "mac"
    )
    // TODO: Is this correct? The test results changed from "@" to "2" after updating packages
  ).toEqual(["2", "Shift-2"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "@",
        keyCode: 64,
        shiftKey: true,
      }),
      "mac"
    )
  ).toEqual(["@", "Shift-@"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: " ",
        keyCode: 32,
      }),
      "mac"
    )
  ).toEqual([" "]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: " ",
        keyCode: 32,
        shiftKey: true,
      }),
      "mac"
    )
  ).toEqual(["Shift- "]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "ArrowUp",
        keyCode: 38,
      }),
      "mac"
    )
  ).toEqual(["ArrowUp"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "ArrowUp",
        keyCode: 38,
        shiftKey: true,
      }),
      "mac"
    )
  ).toEqual(["Shift-ArrowUp"]);

  expect(
    getEventShortcutNames(
      createKeyboardEvent("keypress", {
        key: "Shift",
        keyCode: 16,
        shiftKey: true,
      }),
      "mac"
    )
  ).toEqual(["Shift"]);
});
